from flask import Flask, jsonify
from http.server import HTTPServer, BaseHTTPRequestHandler
from pymongo import MongoClient
from pymongo.server_api import ServerApi
from consts import DB_NAME, COLLECTION, MONGO_URI, MONGO_CERT_PATH, SSL_CERT_PATH, SSL_KEY_PATH, HOST, PORT
from map.get_markers import get_markers
import atexit
import ssl
import keyboard

app = Flask(__name__)
terminate_server = 0

"""
# Initialise MongoDB connection
try:
    client = MongoClient(MONGO_URI,
                         tls=True,
                         tlsCertificateKeyFile=MONGO_CERT_PATH,
                         server_api=ServerApi('1'))

    db = client[DB_NAME]
    collection = db[COLLECTION]
    print("Connected to MongoDB")
except Exception as e:
    print(f"Failed to connect to MongoDB: {e}")
"""
    
# Define HTTP class
class IceHTTP(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == '/':
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()

            self.wfile.write(b"Root path hit!")

        # Update_map endpoint
        elif self.path == '/update_map': # NB: should be POST?
            # Fetch marker data
            markers_data, resp_code = get_markers()

            # Set headers
            self.send_response(resp_code)
            self.send_header("Content-type", "application/json")
            self.end_headers()

            # Write the JSON data to response object
            self.wfile.write(str(markers_data).encode('utf-8'))

# Listen for pressing of q key to terminate server
def on_key_press(server, event):
    if event.name == 'q':
        print('Terminating server...')
        server.server_close()
        keyboard.unhook_all()
        quit()


# Start a server on port 8443 using self defined HTTP class
if __name__ == "__main__":
    print("Starting server on port ", PORT)
    try:
        # Load SSL certificate and private key
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        ssl_context.load_cert_chain(SSL_CERT_PATH, SSL_KEY_PATH)

        # Create HTTP server with SSL support
        server = HTTPServer((HOST, PORT), IceHTTP)
        server.socket = ssl_context.wrap_socket(server.socket, server_side=True)

        print("Server running on port ", PORT)

        # Register key press event handler
        keyboard.on_press(lambda event: on_key_press(server, event))

        # Run server indefinitely
        server.serve_forever()

    except Exception as e:
        print(f"Failed to start server on port {PORT}: {e}")
